#include "Complex.h"
#include <math.h>
double CCPXepsilon=0.0000001;

CComplex::CComplex(void)
{
	re=im=0.0;
}

CComplex::~CComplex(void)
{
}

CComplex::CComplex(double r,double i)
{
	re=r;im=i;
}

CComplex::CComplex(const CComplex& z)
{
	re=z.re;im=z.im;
}

CComplex& CComplex::operator =(const  CComplex& a)
{
	re=a.re;im=a.im;
	return *this;
}

CComplex operator +(const  CComplex& a,const  CComplex& b)
{
	return CComplex(a.re+b.re,a.im+b.im);
}

CComplex operator -(const  CComplex& a,const  CComplex& b)
{
	return CComplex(a.re-b.re,a.im-b.im);
}

CComplex operator *(const  CComplex& a,const  CComplex& b)
{
	return CComplex(a.re*b.re-a.im*b.im,a.re*b.im+a.im*b.re);
}

CComplex operator /(const  CComplex& a,const  CComplex& b)
{
	if(b.re==0.0 && b.im==0.0){
		return CComplex(0.0,0.0);
	}
	double den=b.re*b.re+b.im*b.im;
	return CComplex((a.re*b.re+a.im*b.im)/den,(-a.re*b.im+a.im*b.re)/den);
}

CComplex CComplex::operator +=(const  CComplex& a)
{
	re += a.re;
	im += a.im;
	return *this;
}

CComplex CComplex::operator -=(const  CComplex& a)
{
	re -= a.re;
	im -= a.im;
	return *this;
}

CComplex CComplex::operator *=(const  CComplex& b)
{
	double r=re*b.re-im*b.im;
	double i=re*b.im+im*b.re;
	re=r;im=i;
	return *this;
}
CComplex CComplex::operator /=(const  CComplex& b)
{
	if(b.re==0.0 && b.im==0.0){
		re=0.0;im=0.0;
		return *this;
	}
	double den=b.re*b.re+b.im*b.im;
	double r=(re*b.re+im*b.im)/den;
	double i=(-re*b.im+im*b.re)/den;
	re=r;im=i;
	return *this;
}

bool operator == (const CComplex& a,const CComplex&b)
{
	if(a.re<b.re+CCPXepsilon && a.re>b.re-CCPXepsilon && 
		a.im<b.im+CCPXepsilon && a.im>b.im-CCPXepsilon){
		return true;
	}
	return false;
}

bool operator != (const CComplex& a,const CComplex&b)
{
	if(a.re<b.re+CCPXepsilon && a.re>b.re-CCPXepsilon && 
		a.im<b.im+CCPXepsilon && a.im>b.im-CCPXepsilon){
		return false;
	}
	return true;
}

bool CComplex::zerop()
{
	if(re<CCPXepsilon && re>-CCPXepsilon && 
		im<CCPXepsilon && im>-CCPXepsilon){
		return true;
	}
	return false;
}

bool zerop(CComplex z)
{
	if(z.re<CCPXepsilon && z.re>-CCPXepsilon && 
		z.im<CCPXepsilon && z.im>-CCPXepsilon){
		return true;
	}
	return false;
}


bool CComplex::realp(void)
{
	if(im<CCPXepsilon && im>-CCPXepsilon){
		return true;
	}
	return false;
}

double CComplex::real(void)
{
	return re;
}

double CComplex::imag(void)
{
	return im;
}


void CComplex::SetComplex(double r, double i)
{
	re=r;im=i;
}

double supnorm(CComplex z)
{

	double zre= (z.re>0)? z.re:-z.re;
	double zim= (z.im>0)? z.im:-z.im;
	return zre+zim;
}

double norm(CComplex z)
{
	return sqrt(z.im * z.im + z.re*z.re);
}

double norm2(CComplex z)
{
	return (z.im * z.im + z.re * z.re);
}

CComplex sqrtC(CComplex x)
{
	double th=(atan2((double)(x.imag()),(double)(x.real())))*0.5;
	double nr=sqrt(sqrt(x.real()*x.real()+x.imag()*x.imag()));
	return CComplex(nr*cos(th),nr*sin(th));
}

CComplex conjugate(CComplex x)
{
	return CComplex(x.re,-x.im);
}
