//
//  Evaluate.cpp
//


#include "GConst.h"
#include <math.h>

#include "resource.h"
#ifdef __WXMSW__
	#include "word.h"
#else
	#include "word_e.h"
#endif
#include "incident.h"
#include "wx/wx.h"
#include "wx/string.h"
#include "KidsCindyCanvas.h"
#include "KidsCindyFrame.h"

void Meet_CL(CComplex aa,CComplex bb,CComplex cc, CComplex pp,CComplex qq,CComplex rr,CComplex *xx1 ,CComplex *xx2);
bool FindCnFromPt(GConst* cn,GPoint* pt,GConst** cn1);

bool GConst::Evaluate_FreePoint(void)
{
	if(PInput1==NULL || POutput==NULL){
		return true;
	}
	POutput->xx=para1=PInput1->xx;			
	POutput->yy=para2=PInput1->yy;			
	POutput->zz=para3=PInput1->zz;
	//printf("%s(%.3lf,%.3lf)\n",POutput->_name.c_str(),POutput->xz().real(),POutput->yz().real());
	return false;
}

bool GConst::Evaluate_Join()
{
	if(PInput1==NULL || PInput2==NULL || LOutput==NULL){
		return true;
	}
	LOutput->xx = PInput1->yy * PInput2->zz - PInput1->zz * PInput2->yy;
	LOutput->yy = PInput1->zz * PInput2->xx - PInput1->xx * PInput2->zz;
	LOutput->zz = PInput1->xx * PInput2->yy - PInput1->yy * PInput2->xx;
	if(LOutput->xx==CComplex(0.0,0.0) && LOutput->yy==CComplex(0.0,0.0) && LOutput->zz==CComplex(0.0,0.0)){ 
		LOutput->xx=CComplex(1.0,0.0);
		LOutput->zz=PInput1->xx * CComplex(-1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_PtOnLine()
{
	if(PInput1==NULL || PInput2==NULL || POutput==NULL){
		return true;
	}
	POutput->xx=(PInput1->xz()*para2 + PInput2->xz()*para1);
	POutput->yy=(PInput1->yz()*para2 + PInput2->yz()*para1);
	POutput->zz=para1+para2;
	//printf("%s(%.3lf,%.3lf)",POutput->_name.c_str(),POutput->xz().real(),POutput->yz().real());
	return false;
}

bool GConst::Evaluate_Meet()
{
	if(LInput1==NULL || LInput2==NULL || POutput==NULL){
		return true;
	}
	POutput->xx = LInput1->yy * LInput2->zz - LInput1->zz * LInput2->yy;
	POutput->yy = LInput1->zz * LInput2->xx - LInput1->xx * LInput2->zz;
	POutput->zz = LInput1->xx * LInput2->yy - LInput1->yy * LInput2->xx;
	if(POutput->xx==CComplex(0.0,0.0) && POutput->yy==CComplex(0.0,0.0) && POutput->zz==CComplex(0.0,0.0)){ 
		POutput->xx=CComplex(1.0,0.0);
		POutput->zz=LInput1->xx * CComplex(-1.0,0.0);
	}
	//printf("%s(%.3lf,%.3lf)\n",POutput->_name.c_str(),POutput->xz().real(),POutput->yz().real());
	return false;
}

bool GConst::Evaluate_Parallel()
{
	if(LInput1==NULL || PInput1==NULL || LOutput==NULL){
		return true;
	}
	LOutput->xx=LInput1->xx;
	LOutput->yy=LInput1->yy;
	LOutput->zz=(LInput1->xx * PInput1->xz() + LInput1->yy * PInput1->yz())*CComplex(-1.0,0.0);
	if(LOutput->xx==CComplex(0.0,0.0) && LOutput->yy==CComplex(0.0,0.0) && LOutput->zz==CComplex(0.0,0.0)){ 
		LOutput->xx=CComplex(1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_Perp()
{
	if(LInput1==NULL || PInput1==NULL || LOutput==NULL){
		return true;
	}
	LOutput->xx=LInput1->yy;
	LOutput->yy=LInput1->xx * CComplex(-1.0,0.0);
	LOutput->zz=(LInput1->yy * PInput1->xz() - LInput1->xx * PInput1->yz())*CComplex(-1.0,0.0);
	if(LOutput->xx==CComplex(0.0,0.0) && LOutput->yy==CComplex(0.0,0.0) && LOutput->zz==CComplex(0.0,0.0)){ 
		LOutput->xx=CComplex(1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_Mid()
{
	if(PInput1==NULL || PInput2==NULL || POutput==NULL){
		return true;
	}
	//x1*z2+x2*z1:y1*z2+y2*z1:2*z1*z2
	POutput->xx=PInput1->xx * PInput2->zz + PInput1->zz * PInput2->xx;
	POutput->yy=PInput1->yy * PInput2->zz + PInput1->zz * PInput2->yy;
	POutput->zz=PInput1->zz * PInput2->zz * CComplex(2.0,0.0);
	if(POutput->xx==CComplex(0.0,0.0) && POutput->yy==CComplex(0.0,0.0) && POutput->zz==CComplex(0.0,0.0)){ 
		POutput->xx=CComplex(1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_PtOnPara()
{
	if(PInput1==NULL || LInput1==NULL || POutput==NULL){
		return true;
	}
	POutput->xx=PInput1->xz() - LInput1->yy * para1;
	POutput->yy=PInput1->yz() + LInput1->xx * para1;
	POutput->zz=CComplex(1.0,0.0);
	return false;
}

bool GConst::Evaluate_PtOnPerp()
{
	if(PInput1==NULL || LInput1==NULL || POutput==NULL){
		return true;
	}
	POutput->xx=PInput1->xz() + LInput1->xx * para1;
	POutput->yy=PInput1->yz() + LInput1->yy * para1;
	POutput->zz=CComplex(1.0,0.0);
	return false;
}

bool GConst::Evaluate_MCircle()
{
	if(PInput1==NULL || PInput2==NULL || COutput==NULL){
		return true;
	}
	COutput->xx=PInput1->xz();
	COutput->yy=PInput1->yz();
	COutput->rr=(PInput1->xz()-PInput2->xz())*(PInput1->xz()-PInput2->xz())+(PInput1->yz()-PInput2->yz())*(PInput1->yz()-PInput2->yz());
	return false;
}

bool GConst::Evaluate_PtOnCircle()
{
	if(CInput1==NULL || POutput==NULL){
		return true;
	}
	POutput->xx = CInput1->xx + CComplex(sqrt(CInput1->rr.real()) * cos(para1.real()),para1.imag());
	POutput->yy = CInput1->yy - CComplex(sqrt(CInput1->rr.real()) * sin(para1.real()),para1.imag());
	POutput->zz = CComplex(1.0,0.0);
	//printf("%s(%+.3lf,%+.3lf)(%+.3f)\n",POutput->_name.c_str(),POutput->xz().real(),POutput->yz().real(),para1.real());
	return false;
}

bool GConst::Evaluate_MeetCL()
{
	CComplex xx1,xx2;
	CComplex yy1,yy2;
	double n1,n2,n3,n4,m1,m2;
	if(CInput1==NULL || LInput1==NULL || POutput==NULL){
		return true;
	}
	if(LInput1->yy.zerop()){
		Meet_CL(LInput1->yy,LInput1->xx,LInput1->zz,CInput1->yy,CInput1->xx,CInput1->rr,&yy1,&yy2);
		xx1 = xx2 = CComplex(-1.0,0.0)* LInput1->zz / LInput1->xx;	
	}
	else {
		Meet_CL(LInput1->xx,LInput1->yy,LInput1->zz,CInput1->xx,CInput1->yy,CInput1->rr,&xx1,&xx2);
		yy1 = CComplex(-1.0,0.0)* (LInput1->xx * xx1 + LInput1->zz ) / LInput1->yy;
		yy2 = CComplex(-1.0,0.0)* (LInput1->xx * xx2 + LInput1->zz ) / LInput1->yy;	
	}
	if(norm2(xx1-xx2)+norm2(yy1-yy2)<0.0000001){
		xx1+=CComplex(0.001,0.0);
		yy1+=CComplex(0.001,0.0);
		xx2-=CComplex(0.001,0.0);
		yy2-=CComplex(0.001,0.0);
	}
	if(PInput1!=NULL){
		n1=norm(xx1 - PInput1->xz())+norm(yy1 - PInput1->yz());
		n2=norm(xx2 - PInput1->xz())+norm(yy2 - PInput1->yz());
		if(n1<n2){
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}
		else {
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}
	}
	else if((xx1.realp() && para1.realp()) || (!xx1.realp() && !para1.realp())){
		n1=norm(xx1-para1)+norm(yy1-para2);
		n2=norm(xx2-para1)+norm(yy2-para2);
		n3=(para3==CComplex(999.0,999.0))? 0.0: norm(xx1-para3)+norm(yy1-para4);
		n4=(para4==CComplex(999.0,999.0))? 0.0: norm(xx2-para3)+norm(yy2-para4);
		m1=(n1>n2)? 2.0:1.0;
		m2=(n3>n4)? 2.0:1.0;
		if(m1==m2 && para3!=CComplex(999.0,999.0)){
			n1=(xx1-xx2).real();
			n2=(yy1-yy2).real();
			n3=(para3==CComplex(999.0,999.0))? para1.real() : (para1-para3).real();
			n4=(para4==CComplex(999.0,999.0))? para2.real() : (para2-para4).real();
			m1=atan2(n1,n2);
			m2=atan2(n3,n4);
			if(m1>m2+3.14159){m2+=6.28318;}
			if(m2>m1+3.14159){m1+=6.28318;}
			if(m1-m2<-3.14159 || m1-m2>3.14159){
				POutput->xx = xx1;
				POutput->yy = yy1;
				para1 = xx1;
				para2 = yy1;
				para3 = xx2;
				para4 = yy2;
			}
			else {
				POutput->xx = xx2;
				POutput->yy = yy2;
				para1 = xx2;
				para2 = yy2;
				para3 = xx1;
				para4 = yy1;
			}	
		}
		else if(n1+n4<n2+n3){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	else if(!xx1.realp() && para1.realp()){
		n1=para1.real()-para3.real();
		n2=xx1.imag();
		if(n1*n2>0.0){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	else if(xx1.realp() && !para1.realp()){
		n1=para1.imag();
		n2=xx1.real()-xx2.real();
		if(n1*n2<0.0){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_MeetCC()
{
	CComplex xx1,xx2;
	CComplex yy1,yy2;
	double n1,n2,n3,n4,m1,m2;
	CComplex c1x,c1y,c1r,c2x,c2y,c2r,lx,ly,lz;
	if(CInput1==NULL || CInput2==NULL || POutput==NULL){
		return true;
	}
	c1x=CInput1->xx;c1y=CInput1->yy;c1r=CInput1->rr;
	c2x=CInput2->xx;c2y=CInput2->yy;c2r=CInput2->rr;
	lx=CComplex(2.0,0.0)*(c1x-c2x);
	ly=CComplex(2.0,0.0)*(c1y-c2y);
	lz=c1r-c2r-c1x*c1x-c1y*c1y+c2x*c2x+c2y*c2y;
	if(lx.zerop() && ly.zerop()){
		POutput->xx==CComplex(1.0,0.0);
		POutput->yy==CComplex(1.0,0.0);
		POutput->zz==CComplex(0.0,0.0);
		return true;
	}
	if(ly.zerop()){
		Meet_CL(ly,lx,lz,c1y,c1x,c1r,&yy1,&yy2);
		xx1 = xx2 = CComplex(-1.0,0.0)* lz / lx;	
	}
	else {
		Meet_CL(lx,ly,lz,c1x,c1y,c1r,&xx1,&xx2);
		yy1 = CComplex(-1.0,0.0)* (lx * xx1 + lz ) / ly;
		yy2 = CComplex(-1.0,0.0)* (lx * xx2 + lz ) / ly;	
	}
	if(norm(xx1-xx2)+norm(yy1-yy2)<0.00001){
		xx1+=CComplex(0.001,0.0);
		yy1+=CComplex(0.001,0.0);
		xx2-=CComplex(0.001,0.0);
		yy2-=CComplex(0.001,0.0);
	}
	if(PInput1!=NULL){
		n1=norm(xx1 - PInput1->xz())+norm(yy1 - PInput1->yz());
		n2=norm(xx2 - PInput1->xz())+norm(yy2 - PInput1->yz());
		if(n1<n2){
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}
		else {
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}
	}
	else if((xx1.realp() && para1.realp()) || (!xx1.realp() && !para1.realp())){
		n1=norm(xx1-para1)+norm(yy1-para2);
		n2=norm(xx2-para1)+norm(yy2-para2);
		n3=(para3==CComplex(999.0,999.0))? 0.0: norm(xx1-para3)+norm(yy1-para4);
		n4=(para4==CComplex(999.0,999.0))? 0.0: norm(xx2-para3)+norm(yy2-para4);
		m1=(n1>n2)? 2.0:1.0;
		m2=(n3>n4)? 2.0:1.0;
		if(m1==m2 && para3!=CComplex(999.0,999.0)){
			n1=(xx1-xx2).real();
			n2=(yy1-yy2).real();
			n3=(para3==CComplex(999.0,999.0))? para1.real() : (para1-para3).real();
			n4=(para4==CComplex(999.0,999.0))? para2.real() : (para2-para4).real();
			m1=atan2(n1,n2);
			m2=atan2(n3,n4);
			if(m1>m2+3.14159){m2+=6.28318;}
			if(m2>m1+3.14159){m1+=6.28318;}
			if(m1-m2<-3.14159 || m1-m2>3.14159){
				POutput->xx = xx1;
				POutput->yy = yy1;
				para1 = xx1;
				para2 = yy1;
				para3 = xx2;
				para4 = yy2;
			}
			else {
				POutput->xx = xx2;
				POutput->yy = yy2;
				para1 = xx2;
				para2 = yy2;
				para3 = xx1;
				para4 = yy1;
			}	
		}
		else if(n1+n4<n2+n3){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	else if(!xx1.realp() && para1.realp()){
		n1=para1.real()-para3.real();
		n2=xx1.imag();
		if(n1*n2>0.0){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	else if(xx1.realp() && !para1.realp()){
		n1=para1.imag();
		n2=xx1.real()-xx2.real();
		if(n1*n2<0.0){
			POutput->xx = xx1;
			POutput->yy = yy1;
			para1 = xx1;
			para2 = yy1;
			para3 = xx2;
			para4 = yy2;
		}	
		else {
			POutput->xx = xx2;
			POutput->yy = yy2;
			para1 = xx2;
			para2 = yy2;
			para3 = xx1;
			para4 = yy1;
		}	
		POutput->zz = CComplex(1.0,0.0);
	}
	return false;
}

bool GConst::Evaluate_Bisector()
{
	if(LInput1==NULL || LInput2==NULL || LOutput==NULL){
		return true;
	}
	LOutput->xx = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->xx + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->xx;
	LOutput->yy = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->yy + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->yy;
	LOutput->zz = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->zz + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->zz;
	return false;
}

bool GConst::Evaluate_3pBisector()
{
	double g1x,g1y,g2x,g2y,g3x,g3y,g4x,g4y,norm12,norm32;
	if(PInput1==NULL || PInput2==NULL || PInput3==NULL ||LOutput==NULL){
		return true;
	}
	g1x=PInput1->xz().real();
	g1y=PInput1->yz().real();
	g2x=PInput2->xz().real();
	g2y=PInput2->yz().real();
	g3x=PInput3->xz().real();
	g3y=PInput3->yz().real();
	norm12 = sqrt((g1x-g2x)*(g1x-g2x)+(g1y-g2y)*(g1y-g2y));
	norm32 = sqrt((g3x-g2x)*(g3x-g2x)+(g3y-g2y)*(g3y-g2y));
	if(norm12<0.0001){
		norm12=1.0;
		g1x=g2x+1.0;
		g1y=g2y;
	}
	if(norm32<0.0001){
		norm32=1.0;
		g3x=g2x;
		g3y=g2y+1.0;
	}
	g1x=g2x + (g1x-g2x)/norm12;
	g1y=g2y + (g1y-g2y)/norm12;
	g3x=g2x + (g3x-g2x)/norm32;
	g3y=g2y + (g3y-g2y)/norm32;
	g4x = (g1x+g3x)*0.5;
	g4y = (g1y+g3y)*0.5;
	if(g4x!=0.0 || g4y!=0.0){
		LOutput->xx = CComplex(g2y * 1.0 - 1.0 * g4y,0.0);
		LOutput->yy = CComplex(1.0 * g4x - g2x * 1.0,0.0);
		LOutput->zz = CComplex(g2x * g4y - g2y * g4x,0.0);
	}
	else {
		g4x = g1x-g3x;
		g4y = g1y-g3y;
		LOutput->xx = CComplex(g4x,0.0);
		LOutput->yy = CComplex(g4y,0.0);
		LOutput->zz = CComplex(- g2x * g4x - g2y * g4y,0.0);
	}
	//printf("%+.3lfx%+.3lfy%+.3lf=0:",LOutput->xx.real(),LOutput->yy.real(),LOutput->zz.real());
	return false;
}

bool GConst::Evaluate_Graph()
{
	bool good0,good1;
	double geomstep = geomperdot*10;
	double geomx,geomy,geomx0,geomy0;
	double norm01;
	int previous_error=0;
	bool err;
	if(CurveOutput==NULL){
		return true;
	}
	wxString tgt;
	EvaluateText(CurveOutput->my_cn,CurveOutput->str_y,tgt,true);
	DeleteGCalc(CurveOutput->gcal_y);
	if(MakeCalcPolynomial(tgt,&(CurveOutput->gcal_y))){
		printf("failure\n");
		return true;
	}
	CurveOutput->orbit_len=0;
	geomx0=boxleft-geomstep*3;
	if(CurveOutput->para_min>geomx0){
		geomx0=CurveOutput->para_min;
	}
	CurveOutput->gcal_y->Evaluate(geomx0,&geomy0);
	good0 = false;
	for(geomx = geomx0+geomstep;geomx<boxright && geomx<CurveOutput->para_max+geomstep;geomx +=geomstep){
		if(geomx > CurveOutput->para_max){
			geomx=CurveOutput->para_max;
		}
		do{
			err=false;
			good1 = CurveOutput->gcal_y->Evaluate(geomx,&geomy);
			good1 &= (geomy0>boxbottom-(boxtop-boxbottom)*0.1 && geomy0<boxtop+(boxtop-boxbottom)*0.1);
			if(!good1){
				geomstep=geomperdot*10;
				previous_error=0;
			}
			else {
				norm01=norm(CComplex(geomx-geomx0,geomy-geomy0));
				if(norm01>geomperdot*10 && geomstep>0.0005 && previous_error!=2){
					geomx -= geomstep;
					geomstep *= 0.7;
					geomx += geomstep;
					err=true;
					previous_error=1;
				}
				else if(norm01<geomperdot*5 && geomstep<geomperdot*20 && previous_error!=1){
					geomx -= geomstep;
					geomstep /= 0.7;
					geomx += geomstep;
					err=true;
					previous_error=2;
				}
				else {
					previous_error=0;
				}
			}
		} while (err);
		if(good0 && good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		else if(!good0 && good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx0;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy0;
				CurveOutput->orbit_len++;
			}
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		else if(good0 && !good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		good0=good1;
		geomx0=geomx;
		geomy0=geomy;
	}
	return false;
}

bool GConst::Evaluate_PtOnCurve()
{
	if(CurveInput1==NULL || POutput==NULL){
		return true;
	}
	double dda=para1.real();
	double dd1,dd2,dd3,dd4;
	int floordd=(int)floor(dda);
	double dd0 = dda - floor(dda);
	if(floordd<0 || floordd>=CurveInput1->orbit_len - 1){
		floordd=0;
	}
	//printf("dda=%lf,floordd=%d,dd0=%lf\n",dda,floordd,dd0);
	dd1=CurveInput1->orbitx[floordd];
	dd2=CurveInput1->orbity[floordd];
	dd3=CurveInput1->orbitx[floordd+1];
	dd4=CurveInput1->orbity[floordd+1];
	POutput->xx=CComplex((1.0-dd0)*dd1+dd0*dd3,0.0);
	POutput->yy=CComplex((1.0-dd0)*dd2+dd0*dd4,0.0);	
	POutput->zz=CComplex(1.0,0.0);
	return false;
}

bool GConst::Evaluate_Tangent()
{
	double px,py,cx,cy,r2,pz;
	px=PInput1->xz().real();
	py=PInput1->yz().real();
	cx=CInput1->xx.real();
	cy=CInput1->yy.real();
	r2=CInput1->rr.real();
	pz=-r2-cx*(px-cx)-cy*(py-cy);
	px -= cx;
	py -= cy;
	LOutput->xx=CComplex(px,0.0);
	LOutput->yy=CComplex(py,0.0);	
	LOutput->zz=CComplex(pz,0.0);
	return false;
}

bool GConst::Evaluate_Compass()
{
	CComplex px1,py1,px2,py2,pr;
	px1=PInput1->xz();
	py1=PInput1->yz();
	px2=PInput2->xz();
	py2=PInput2->yz();
	pr=(px1-px2)*(px1-px2)+(py1-py2)*(py1-py2);
	COutput->xx=PInput3->xz();
	COutput->yy=PInput3->yz();	
	COutput->rr=pr;
	return false;
}

bool GConst::Evaluate_Orbit1()
{
	double loopstep,loopcursor;
	GConst *mycns,*mystartcn,*myendcn,*cnp;
	CComplex oldpara1,oldpara2;
	bool err,ok;
	short previous_error=0;
	double g1x,g1y,g1xi,g1yi,g2x,g2y,g2xi,g2yi;
	CComplex branch_para1,branch_para2,branch_para3,branch_para4;
	double norm12;
	bool doublebranched=false;
	GConst* doublebranch_cn=NULL;
	//ptsx=makePClone(pts);
	//copyToPClone(ptsx,pts); // may no use
	//initialize
	mycns=CurveOutput->my_cn;
	CurveOutput->orbit_len=0;
	FindCnFromPt(mycns,PInput1,&mystartcn);
	FindCnFromPt(mycns,PInput2,&myendcn);
	if(mystartcn==NULL || myendcn==NULL){
		return true;
	}
	oldpara1 = mystartcn->para1;
	g1x=PInput2->xz().real();
	g1y=PInput2->yz().real();
	g1xi=PInput2->xz().imag();
	g1yi=PInput2->yz().imag();
	if(g1xi<=-0.000001 || g1xi >=0.000001 || g1yi<=-0.000001 || g1yi>=0.000001){
		return false;
	}
	// check double branched or not
	for(cnp=mycns;cnp!=NULL && cnp!=myendcn;cnp=cnp->next){
		if((cnp->Const_Type==CONST_MEET_CL || cnp->Const_Type==CONST_MEET_CC) && cnp->PInput1==NULL){
			doublebranched=true;
			doublebranch_cn=cnp;
		}
	}
	// main loop
	loopstep=0.2;
	short phase=0;
	bool main_loop_continue=true;
	short phase_sign=1;
	for(loopcursor=oldpara1.real()+loopstep;main_loop_continue;loopcursor+=(loopstep*phase_sign)){
		do{
			err=false;
			if(doublebranched){
				branch_para1=doublebranch_cn->para1;
				branch_para2=doublebranch_cn->para2;
				branch_para3=doublebranch_cn->para3;
				branch_para4=doublebranch_cn->para4;
			}
			mystartcn->para1=CComplex(loopcursor,0.0);
			for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
				cnp->Evaluate();
			}
			cnp->Evaluate();
			g2x=PInput2->xz().real();
			g2y=PInput2->yz().real();
			g2xi=PInput2->xz().imag();
			g2yi=PInput2->yz().imag();
			if(g2x>boxleft && g2x<boxright && g2y>boxbottom && g2y<boxtop){
				norm12 = norm(CComplex(g2x-g1x,g2y-g1y));
				if(norm12>geomperdot*10  && loopstep>0.0005 && previous_error!=2){
					loopcursor -= (loopstep*phase_sign);
					loopstep *= 0.7;
					loopcursor += (loopstep*phase_sign);
					err=true;
					previous_error=1;
					if(doublebranched){
						doublebranch_cn->para1=branch_para1;
						doublebranch_cn->para2=branch_para2;
						doublebranch_cn->para3=branch_para3;
						doublebranch_cn->para4=branch_para4;
					}
				}
				else if(norm12<geomperdot*5  && loopstep<0.2  && previous_error!=1){
					loopcursor -= (loopstep*phase_sign);
					loopstep /= 0.7;
					loopcursor += (loopstep*phase_sign);
					err=true;
					previous_error=2;
					if(doublebranched){
						doublebranch_cn->para1=branch_para1;
						doublebranch_cn->para2=branch_para2;
						doublebranch_cn->para3=branch_para3;
						doublebranch_cn->para4=branch_para4;
					}
				}
				else {
					previous_error=0;
				}
			}
			else {
				loopstep=0.2;
				previous_error=0;
			}
		} while(err);
		ok=false;
		if(CurveOutput->orbit_len==0){
			ok=true;
		}
		else{
			norm12 = norm(CComplex(g2x-CurveOutput->orbitx[CurveOutput->orbit_len-1],g2y-CurveOutput->orbity[CurveOutput->orbit_len-1]));
			if(norm12>geomperdot*7.5){
				ok=true;
			}
			if(g1xi>=-0.000001 && g1xi<=0.000001 && (g2xi<-0.000001 || g2xi>0.000001)){
				ok=true;
				if(phase==0){
					phase = 1;
					phase_sign=-1;
				}
				else if(phase==1){
					phase=2;
					phase_sign=1;
				}
			}
		}
		if(ok && CurveOutput->orbit_len<500){
			CurveOutput->orbitx[CurveOutput->orbit_len]=g2x;
			CurveOutput->orbity[CurveOutput->orbit_len]=g2y;
			CurveOutput->orbit_real[CurveOutput->orbit_len]=(g2xi==0.0 && g2yi==0.0);
			CurveOutput->orbit_len ++;
		}
		g1x=g2x;g1y=g2y;
		g1xi=g2xi;g1yi=g2yi;
		if(phase==0 && loopcursor >= 6.28+oldpara1.real()){
			main_loop_continue=false;
		}
		else if(phase ==1 && loopcursor <= oldpara1.real()-6.28){
			main_loop_continue=false;
		}
		else if(phase ==2 && loopcursor>=oldpara1.real()){
			main_loop_continue=false;
		}
	}
	mystartcn->para1=oldpara1;
	if(doublebranched){
		for(cnp=mycns;cnp!=NULL && cnp!=doublebranch_cn;cnp=cnp->next){
			cnp->Evaluate();
		}
		doublebranch_cn->para1=branch_para1;
		doublebranch_cn->para2=branch_para2;
		doublebranch_cn->para3=branch_para3;
		doublebranch_cn->para4=branch_para4;
		for(cnp=doublebranch_cn;cnp!=NULL && cnp!=myendcn;cnp=cnp->next){
			cnp->Evaluate();
		}
	}
	else {
		for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
			cnp->Evaluate();
		}
	}
	myendcn->Evaluate();
	return false;
}

bool GConst::Evaluate_Orbit2()
{
	double loopstep,loopcursor;
	GConst *mycns,*mystartcn,*myendcn,*cnp;
	GConst* doublebranch_cn=NULL;
	CComplex oldpara1,oldpara2;
	bool err,doublebranched=false;
	double g1x,g1y,g1xi,g1yi,g2x,g2y,g2xi,g2yi;
	double mystartpara;
	double norm12;
	loopstep=0.2;
	mycns=CurveOutput->my_cn;
	CurveOutput->orbit_len=0;
	FindCnFromPt(mycns,PInput1,&mystartcn);
	FindCnFromPt(mycns,PInput2,&myendcn);
	if(mystartcn==NULL || myendcn==NULL){
		return true;
	}
	oldpara1 = mystartcn->para1;
	oldpara2 = mystartcn->para2;
	mystartpara=oldpara1.real()/(oldpara1.real()+oldpara2.real());
	//initialize
	g1x=PInput2->xz().real();
	g1y=PInput2->yz().real();
	g1xi=PInput2->xz().imag();
	g1yi=PInput2->yz().imag();
	if(g1xi<=-0.000001 || g1xi >=0.000001 || g1yi<=-0.000001 || g1yi>=0.000001){
		return false;
	}
	// check double branched or not
	for(cnp=mycns;cnp!=NULL && cnp!=myendcn;cnp=cnp->next){
		if((cnp->Const_Type==CONST_MEET_CL || cnp->Const_Type==CONST_MEET_CC) && cnp->PInput1==NULL){
			doublebranched=true;
			doublebranch_cn=cnp;
		}
	}
	bool main_loop_continue=true;
	short phase=0;
	short phase_sign=1;
	CComplex branch_para1,branch_para2,branch_para3,branch_para4;
	short previous_error=0;
	for(loopcursor=mystartpara+loopstep;main_loop_continue;loopcursor+=(loopstep*phase_sign)){
		if(loopcursor>10.0){ 
			for(;loopcursor>-9.5;loopcursor -= 0.2){
				mystartcn->para1=CComplex(loopcursor,0.0);
				mystartcn->para2=CComplex(1.0,0.0) - CComplex(loopcursor,0.0);
				for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
					cnp->Evaluate();
				}
				if(cnp!=NULL){
					cnp->Evaluate();
				}
			}
		}
		else if(loopcursor<-10.0){
			for(;loopcursor<9.5;loopcursor += 0.2){
				mystartcn->para1=CComplex(loopcursor,0.0);
				mystartcn->para2=CComplex(1.0,0.0) - CComplex(loopcursor,0.0);
				for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
					cnp->Evaluate();
				}
				if(cnp!=NULL){
					cnp->Evaluate();
				}
			}
		}
		//printf("loopcursor = %lf : phase = %d\n",loopcursor,phase);
		do{
			err=false;
			if(doublebranched){
				branch_para1=doublebranch_cn->para1;
				branch_para2=doublebranch_cn->para2;
				branch_para3=doublebranch_cn->para3;
				branch_para4=doublebranch_cn->para4;
			}
			mystartcn->para1=CComplex(loopcursor,0.0);
			mystartcn->para2=CComplex(1.0,0.0) - CComplex(loopcursor,0.0);
			for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
				cnp->Evaluate();
			}
			if(cnp!=NULL){
				cnp->Evaluate();
			}
			g2x=PInput2->xz().real();
			g2y=PInput2->yz().real();
			g2xi=PInput2->xz().imag();
			g2yi=PInput2->yz().imag();
			if(g2x>boxleft && g2x<boxright && g2y>boxbottom && g2y<boxtop){
				norm12 = norm(CComplex(g2x-g1x,g2y-g1y));
				if(norm12>geomperdot*4  && loopstep>0.0005 && previous_error!=2){
					loopcursor -= (loopstep*phase_sign);
					loopstep *= 0.7;
					loopcursor += (loopstep*phase_sign);
					previous_error=1;
					if(doublebranched){
						doublebranch_cn->para1=branch_para1;
						doublebranch_cn->para2=branch_para2;
						doublebranch_cn->para3=branch_para3;
						doublebranch_cn->para4=branch_para4;
					}
					err=true;
				}
				else if(norm12<geomperdot  && loopstep<0.2 && previous_error!=1){
					loopcursor -= (loopstep*phase_sign);
					loopstep /= 0.7;
					loopcursor += (loopstep*phase_sign);
					previous_error=2;
					if(doublebranched){
						doublebranch_cn->para1=branch_para1;
						doublebranch_cn->para2=branch_para2;
						doublebranch_cn->para3=branch_para3;
						doublebranch_cn->para4=branch_para4;
					}
					err=true;
				}
				else {
					previous_error=0;
				}
			}
			else {
				loopstep=0.2;
			}
		} while(err);
		bool ok=false;
		if(CurveOutput->orbit_len==0){
			ok=true;
		}
		else{
			norm12 = norm(CComplex(g2x-CurveOutput->orbitx[CurveOutput->orbit_len-1],g2y-CurveOutput->orbity[CurveOutput->orbit_len-1]));
			if(norm12>geomperdot*7.5){
				ok=true;
			}
			if(g1xi>=-0.000001 && g1xi<=0.000001 && (g2xi<-0.000001 || g2xi>0.000001)){
				ok=true;
				if(phase==0){
					phase = 1;
					phase_sign=-1;
				}
				else if(phase==1){
					phase=2;
					phase_sign=1;
				}
			}
		}
		if(ok && CurveOutput->orbit_len<500){
			if(PInput2->xz().imag()==0.0 && PInput2->yz().imag()==0.0){
				CurveOutput->orbitx[CurveOutput->orbit_len]=g2x;
				CurveOutput->orbity[CurveOutput->orbit_len]=g2y;
				CurveOutput->orbit_real[CurveOutput->orbit_len]=(g2xi==0.0 && g2yi==0.0);
				CurveOutput->orbit_len ++;
			}
		}
		g1x=g2x;g1y=g2y;
		g1xi=g2xi;g1yi=g2yi;
		if(phase==0 && mystartpara-loopstep*0.6<=loopcursor && loopcursor<=mystartpara+loopstep*0.6){
			main_loop_continue=false;
		}
		else if(phase ==1 && loopcursor <= -10.00){
			//main_loop_continue=false;
		}
		else if(phase ==2 && mystartpara-loopstep*0.6<=loopcursor && loopcursor<=mystartpara+loopstep*0.6){
			main_loop_continue=false;
		}
	}
	mystartcn->para1=oldpara1;
	mystartcn->para2=oldpara2;
	for(cnp=mycns;cnp!=NULL && cnp->POutput!=PInput2;cnp=cnp->next){
		cnp->Evaluate();
	}
	if(cnp!=NULL){
		cnp->Evaluate();
	}
	return false;
}

bool GConst::Evaluate_Orthcenter()
{
	CComplex px1,py1,px2,py2,px3,py3;
	if(PInput1==NULL || PInput2==NULL || PInput3==NULL || POutput==NULL){
		return true;
	}
	px1=PInput1->xz();
	py1=PInput1->yz();
	px2=PInput2->xz();
	py2=PInput2->yz();
	px3=PInput3->xz();
	py3=PInput3->yz();
	POutput->xx = (px2*px2-px1*px1+py2*py2-py1*py1)*(py3-py1)-(px3*px3-px1*px1+py3*py3-py1*py1)*(py2-py1);
	POutput->yy = (px3*px3-px1*px1+py3*py3-py1*py1)*(px2-px1)-(px2*px2-px1*px1+py2*py2-py1*py1)*(px3-px1);
	POutput->zz = CComplex(2.0,0.0)*((px2-px1)*(py3-py1)-(px3-px1)*(py2-py1));
	return false;
}

bool GConst::Evaluate_PtOn3PBisector()
{
	if(PInput1==NULL || PInput2==NULL || PInput3==NULL || POutput==NULL){
		return true;
	}
	double g1x=PInput1->xz().real();
	double g1y=PInput1->yz().real();
	double g2x=PInput2->xz().real();
	double g2y=PInput2->yz().real();
	double g3x=PInput3->xz().real();
	double g3y=PInput3->yz().real();
	double norm12 = sqrt((g1x-g2x)*(g1x-g2x)+(g1y-g2y)*(g1y-g2y));
	double norm32 = sqrt((g3x-g2x)*(g3x-g2x)+(g3y-g2y)*(g3y-g2y));
	if(norm12<0.0001){
		norm12=1.0;
		g1x=g2x+1.0;
		g1y=g2y;
	}
	if(norm32<0.0001){
		norm32=1.0;
		g3x=g2x;
		g3y=g2y+1.0;
	}
	g1x=g2x + (g1x-g2x)/norm12;
	g1y=g2y + (g1y-g2y)/norm12;
	g3x=g2x + (g3x-g2x)/norm32;
	g3y=g2y + (g3y-g2y)/norm32;
	double g4x = (g1x+g3x)*0.5 - g2x;
	double g4y = (g1y+g3y)*0.5 - g2y;
	double norm4 = sqrt(g4x*g4x+g4y*g4y);
	if(norm4!=0){
		g4x/=norm4;
		g4y/=norm4;
		//printf("g4=(%lf,%lf)\n",g4x,g4y);
	}
	else {
		g4x = g1y-g3y;
		g4y = - g1x+g3x;
		norm4 = sqrt(g4x*g4x+g4y*g4y);
		g4x/=norm4;
		g4y/=norm4;
	}
	//printf("g2=(%lf,%lf),%lf\n",g2x,g2y,para1.real());
	POutput->xx=CComplex(g2x+g4x * para1.real(),0.0);
	POutput->yy=CComplex(g2y+g4y * para1.real(),0.0);
	POutput->zz=CComplex(1.0,0.0);
	//printf("%s(%.3lf,%.3lf)\n",POutput->_name.c_str(),POutput->xz().real(),POutput->yz().real());
	return false;
}

bool GConst::Evaluate_PtOnBisector()
{
	if(LInput1==NULL || LInput2==NULL || POutput==NULL){
		return true;
	}
	CComplex g1x = LInput1->yy * LInput2->zz -  LInput1->zz * LInput2->yy;
	CComplex g1y = LInput1->zz * LInput2->xx -  LInput1->xx * LInput2->zz;
	CComplex g1z = LInput1->xx * LInput2->yy -  LInput1->yy * LInput2->xx;
	CComplex g2x = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->xx + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->xx;
	CComplex g2y = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->yy + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->yy;
	CComplex g2z = sqrtC(LInput2->xx * LInput2->xx + LInput2->yy * LInput2->yy) * LInput1->zz + para1 * sqrtC(LInput1->xx * LInput1->xx + LInput1->yy * LInput1->yy) * LInput2->zz;
	CComplex nn = sqrtC(g2x * g2x + g2y * g2y);
	g2x /= nn;
	g2y /= nn;
	POutput->xx=g1x / g1z + g2y * para2;
	POutput->yy=g1y / g1z + CComplex(-1.0,0.0) * g2x * para2;
	POutput->zz=CComplex(1.0,0.0);

	return false;
}

bool GConst::Evaluate_PtOnTangent()
{
	if(CInput1==NULL || PInput1==NULL || POutput==NULL){
		return true;
	}
	CComplex t1y = PInput1->xz() - CInput1->xx;
	CComplex t1x = CInput1->yy - PInput1->yz();

	POutput->xx= PInput1->xx + t1x * para1 * PInput1->zz;
	POutput->yy= PInput1->yy + t1y * para1 * PInput1->zz;
	POutput->zz= PInput1->zz;

	return false;
}

bool GConst::Evaluate_CurveTangent()
{
	double px,py,mx,my,norm;
	double qx,qy,rx,ry;
	GConst* mycns=CurveInput1->my_cn;
	GConst*cnp;
	if(!FindPtFromCn(mycns,PInput1,&cnp)){
		printf("NG\n");
		return true;
	}
	double para=cnp->para1.real();
	px=PInput1->xz().real();
	py=PInput1->yz().real();
	if(para<1){
		mx=CurveInput1->orbitx[1]-CurveInput1->orbitx[0];
		my=CurveInput1->orbity[1]-CurveInput1->orbity[0];
	}
	else if(para>=CurveInput1->orbit_len-2){
		mx=CurveInput1->orbitx[CurveInput1->orbit_len-1]-CurveInput1->orbitx[CurveInput1->orbit_len-2];
		my=CurveInput1->orbity[CurveInput1->orbit_len-1]-CurveInput1->orbity[CurveInput1->orbit_len-2];
	}
	else{
		int fl=(int)floor(para);
		para = para - 1.0*fl;
		if(para<0.5){
			qx=CurveInput1->orbitx[fl+1];
			qy=CurveInput1->orbity[fl+1];
			rx=CurveInput1->orbitx[fl-1]*(1-2*para)+CurveInput1->orbitx[fl]*2*para;
			ry=CurveInput1->orbity[fl-1]*(1-2*para)+CurveInput1->orbity[fl]*2*para;
		}
		else {
			qx=CurveInput1->orbitx[fl+1]*(2-2*para)+CurveInput1->orbitx[fl+2]*(2*para-1);
			qy=CurveInput1->orbity[fl+1]*(2-2*para)+CurveInput1->orbity[fl+2]*(2*para-1);
			rx=CurveInput1->orbitx[fl];
			ry=CurveInput1->orbity[fl];
		}
		mx=qx-rx;
		my=qy-ry;
	}
	norm=sqrt(mx*mx+my*my);
	mx /= norm;
	my /= norm;
	LOutput->xx=CComplex(my,0.0);
	LOutput->yy=CComplex(-mx,0.0);	
	LOutput->zz=CComplex(-my*px+mx*py,0.0);
	return false;
}

bool GConst::Evaluate_PtOnCurveTangent()
{
	GConst* cnp;
	FindCurveTangentFromCns(CurveInput1->my_cn,CurveInput1,PInput1,&cnp);
	//cnp->Evaluate_CurveTangent();
	double px=PInput1->xz().real();
	double py=PInput1->yz().real();
	POutput->xx=CComplex(px+para1.real()*cnp->LOutput->yy.real(),0.0);
	POutput->yy=CComplex(py-para1.real()*cnp->LOutput->xx.real(),0.0);
	POutput->zz=CComplex(1.0,0.0);
	return false;
}
bool GConst::Evaluate_ParaCurve()
{
	bool good0,good1;
	double parastep = 0.1;
	double geomx,geomy,geomx0,geomy0;
	double para,para0;
	double norm01;
	int previous_error=0;
	bool err;
	if(CurveOutput==NULL){
		return true;
	}
	wxString tgt;
	EvaluateText(CurveOutput->my_cn,CurveOutput->str_x,tgt,true);
	DeleteGCalc(CurveOutput->gcal_x);
	if(MakeCalcPolynomial(tgt,&(CurveOutput->gcal_x))){
		printf("failure\n");
		return true;
	}
	EvaluateText(CurveOutput->my_cn,CurveOutput->str_y,tgt,true);
	DeleteGCalc(CurveOutput->gcal_y);
	if(MakeCalcPolynomial(tgt,&(CurveOutput->gcal_y))){
		printf("failure\n");
		return true;
	}
	CurveOutput->orbit_len=0;
	para0=CurveOutput->para_min-parastep;
	good0 =  CurveOutput->gcal_x->Evaluate(para0,&geomx0);
	good0 &= CurveOutput->gcal_y->Evaluate(para0,&geomy0);
	good0 &= (geomx0>boxleft-(boxright-boxleft)*0.1 && geomx0<boxright+(boxright-boxleft)*0.1);
	good0 &= (geomy0>boxbottom-(boxtop-boxbottom)*0.1 && geomy0<boxtop+(boxtop-boxbottom)*0.1);
	for(para = CurveOutput->para_min;para<CurveOutput->para_max+parastep;para += parastep){
		if(para > CurveOutput->para_max){
			para=CurveOutput->para_max;
		}
		do{
			err=false;
			good1 =  CurveOutput->gcal_x->Evaluate(para,&geomx);
			good1 &= CurveOutput->gcal_y->Evaluate(para,&geomy);
			good1 &= (geomx>boxleft-(boxright-boxleft)*0.1 && geomx<boxright+(boxright-boxleft)*0.1);
			good1 &= (geomy>boxbottom-(boxtop-boxbottom)*0.1 && geomy<boxtop+(boxtop-boxbottom)*0.1);
			if(!good1){
				parastep=0.1;
				previous_error=0;
			}
			else {
				norm01=norm(CComplex(geomx-geomx0,geomy-geomy0));
				if(norm01>geomperdot*10 && parastep>0.0005 && previous_error!=2){
					para -= parastep;
					parastep *= 0.7;
					para += parastep;
					err=true;
					previous_error=1;
				}
				else if(norm01<geomperdot*5 && parastep<0.2 && previous_error!=1){
					para -= parastep;
					parastep /= 0.7;
					para += parastep;
					err=true;
					previous_error=2;
				}
				else {
					previous_error=0;
				}
			}
		} while (err);
		if(good0 && good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		else if(!good0 && good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx0;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy0;
				CurveOutput->orbit_len++;
			}
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		else if(good0 && !good1){
			if(CurveOutput->orbit_len<499){
				CurveOutput->orbitx[CurveOutput->orbit_len]=geomx;
				CurveOutput->orbity[CurveOutput->orbit_len]=geomy;
				CurveOutput->orbit_len++;
			}
		}
		good0=good1;
		para0=para;
		geomx0=geomx;
		geomy0=geomy;
	}
	return false;
}

